@@ -1,4 +1,4 @@
-NNS.M.reg <- function (X_n, Y, factor.2.dummy = FALSE, order = NULL, stn = NULL, n.best = NULL, type = NULL, point.est = NULL, plot = FALSE, residual.plot = TRUE, location = NULL, noise.reduction = 'off', dist = "L2", return.values = FALSE, plot.regions = FALSE, ncores=ncores){
+NNS.M.reg <- function (X_n, Y, factor.2.dummy = FALSE, order = NULL, stn = NULL, n.best = NULL, type = NULL, point.est = NULL, plot = FALSE, residual.plot = TRUE, location = NULL, noise.reduction = 'off', dist = "L2", return.values = FALSE, plot.regions = FALSE, ncores=NULL){
 
 
   ### For Multiple regressions
@@ -80,18 +80,38 @@ NNS.M.reg <- function (X_n, Y, factor.2.dummy = FALSE, order = NULL, stn = NULL,
     reg.points.matrix <- unique(reg.points.matrix)
   }
 
+  if(!is.null(order) && order=="max" && is.null(n.best)) n.best <- 1
+
   ### Find intervals in regression points for each variable, use left.open T and F for endpoints.
   NNS.ID <- list()
 
-  for(j in 1:n){
-    sorted.reg.points <- sort(reg.points.matrix[ , j])
-    sorted.reg.points <- sorted.reg.points[!is.na(sorted.reg.points)]
+  ### PARALLEL
 
-    NNS.ID[[j]] <- findInterval(original.IVs[ , j], sorted.reg.points, left.open = FALSE)
+  if (is.null(ncores)) {
+    num_cores <- as.integer(detectCores()) - 1
+  } else {
+    num_cores <- ncores
   }
 
+  if(num_cores>1){
+    cl <- makeCluster(num_cores)
+    registerDoParallel(cl)
+  } else { cl <- NULL }
 
-  NNS.ID <- do.call(cbind,NNS.ID)
+  if(is.null(cl)){
+
+      for(j in 1:n){
+          sorted.reg.points <- na.omit(sort(reg.points.matrix[ , j]))
+          NNS.ID[[j]] <- findInterval(original.IVs[ , j], vec = sorted.reg.points, left.open = FALSE)
+      }
+  } else {
+      NNS.ID <- foreach(j = 1:n)%dopar%{
+          sorted.reg.points <- na.omit(sort(reg.points.matrix[ , j]))
+          return(findInterval(original.IVs[ , j], vec = sorted.reg.points, left.open = FALSE))
+    }
+  }
+
+  NNS.ID <- do.call(cbind, NNS.ID)
 
   ### Create unique identifier of each observation's interval
   NNS.ID <- gsub(do.call(paste, as.data.frame(NNS.ID)), pattern = " ", replacement = ".")
@@ -105,22 +125,22 @@ NNS.M.reg <- function (X_n, Y, factor.2.dummy = FALSE, order = NULL, stn = NULL,
   data.table::setkey(mean.by.id.matrix, 'NNS.ID', 'obs')
   if(is.numeric(order) || is.null(order)){
     if(noise.reduction == 'off'){
-      mean.by.id.matrix = mean.by.id.matrix[ , y.hat := gravity(original.DV), by = 'NNS.ID']
+      mean.by.id.matrix <- mean.by.id.matrix[ , c(paste("RPM", 1:n), "y.hat") := lapply(.SD, function(z) gravity(as.numeric(z))), .SDcols = seq_len(n+1) ,by = 'NNS.ID']
     }
     if(noise.reduction == 'mean'){
-      mean.by.id.matrix = mean.by.id.matrix[ , y.hat := mean(original.DV), by = 'NNS.ID']
+      mean.by.id.matrix <- mean.by.id.matrix[ , c(paste("RPM", 1:n), "y.hat") := lapply(.SD, function(z) mean(as.numeric(z))), .SDcols = seq_len(n+1), by = 'NNS.ID']
     }
     if(noise.reduction == 'median'){
-      mean.by.id.matrix = mean.by.id.matrix[ , y.hat := median(original.DV), by = 'NNS.ID']
+      mean.by.id.matrix <- mean.by.id.matrix[ , c(paste("RPM", 1:n), "y.hat") := lapply(.SD, function(z) median(as.numeric(z))), .SDcols = seq_len(n+1), by = 'NNS.ID']
     }
     if(noise.reduction == 'mode' & is.null(type)){
-      mean.by.id.matrix = mean.by.id.matrix[ , y.hat := mode(original.DV), by = 'NNS.ID']
+      mean.by.id.matrix <- mean.by.id.matrix[ , c(paste("RPM", 1:n), "y.hat") := lapply(.SD, function(z) mode(as.numeric(z))), .SDcols = seq_len(n+1), by = 'NNS.ID']
     }
     if(noise.reduction == 'mode' & !is.null(type)){
-      mean.by.id.matrix = mean.by.id.matrix[ , y.hat := mode_class(original.DV), by = 'NNS.ID']
+      mean.by.id.matrix <- mean.by.id.matrix[ , c(paste("RPM", 1:n), "y.hat") := lapply(.SD, function(z) mode_class(as.numeric(z))), .SDcols = seq_len(n+1), by = 'NNS.ID']
     }
   } else {
-    mean.by.id.matrix = mean.by.id.matrix[ , y.hat := original.DV, by = 'NNS.ID']
+    mean.by.id.matrix <- mean.by.id.matrix[ , c(paste("RPM", 1:n), "y.hat") := .SD , .SDcols = seq_len(n+1), by = 'NNS.ID']
   }
 
   y.identifier <- mean.by.id.matrix[ , NNS.ID]
@@ -129,45 +149,51 @@ NNS.M.reg <- function (X_n, Y, factor.2.dummy = FALSE, order = NULL, stn = NULL,
   ###Order y.hat to order of original Y
   resid.plot <- mean.by.id.matrix[]
   data.table::setkey(resid.plot, 'obs')
+
+
   y.hat <- mean.by.id.matrix[ , .(y.hat)]
+
   if(!is.null(type)){
     y.hat <- round(y.hat)
   }
 
-
   fitted.matrix <- data.table::data.table(original.IVs, y = original.DV, y.hat, mean.by.id.matrix[ , .(NNS.ID)])
 
+
   data.table::setkey(mean.by.id.matrix, 'NNS.ID')
-  REGRESSION.POINT.MATRIX <- mean.by.id.matrix[ , obs := NULL]
+  REGRESSION.POINT.MATRIX <- mean.by.id.matrix[ , c("obs") := NULL]
+
+  REGRESSION.POINT.MATRIX <- REGRESSION.POINT.MATRIX[, .SD[1], by = NNS.ID]
+
+  REGRESSION.POINT.MATRIX <- REGRESSION.POINT.MATRIX[, .SD, .SDcols = colnames(mean.by.id.matrix)%in%c(paste("RPM", 1:n), "y.hat")]
 
 
-  if(is.numeric(order) || is.null(order)){
-    if(noise.reduction == 'off'){
-      REGRESSION.POINT.MATRIX <- REGRESSION.POINT.MATRIX[ , lapply(.SD, function(z) as.numeric(gravity(z))), by = NNS.ID]
-    }
-    if(noise.reduction == 'mean'){
-      REGRESSION.POINT.MATRIX <- REGRESSION.POINT.MATRIX[ , lapply(.SD, function(z) as.numeric(mean(z))), by = NNS.ID]
-    }
-    if(noise.reduction == 'median'){
-      REGRESSION.POINT.MATRIX <- REGRESSION.POINT.MATRIX[, lapply(.SD, function(z) as.numeric(median(z))), by = NNS.ID]
-    }
-    if(noise.reduction == 'mode' & is.null(type)){
-      REGRESSION.POINT.MATRIX <- REGRESSION.POINT.MATRIX[, lapply(.SD, function(z) as.numeric(mode(z))), by = NNS.ID]
-    }
-    if(noise.reduction == 'mode' & !is.null(type)){
-      REGRESSION.POINT.MATRIX <- REGRESSION.POINT.MATRIX[, lapply(.SD, function(z) as.numeric(mode_class(z))), by = NNS.ID]
+  data.table::setnames(REGRESSION.POINT.MATRIX, 1:n, colnames(mean.by.id.matrix)[1:n])
+
+  if(is.character(n.best)){
+      n.best <- REGRESSION.POINT.MATRIX[ , .N]
+  } else {
+    if(is.null(n.best)){
+      n.best <- floor(sqrt(REGRESSION.POINT.MATRIX[ , .N]))
     }
   }
 
-  REGRESSION.POINT.MATRIX <- REGRESSION.POINT.MATRIX[ , NNS.ID := NULL]
-  REGRESSION.POINT.MATRIX <- REGRESSION.POINT.MATRIX[ , original.DV := NULL]
 
 
-  if(is.character(n.best)){
-    n.best <- REGRESSION.POINT.MATRIX[ , .N]
-  } else {
-    if(is.null(n.best)){
-        n.best <- floor(sqrt(REGRESSION.POINT.MATRIX[ , .N]))
+  if(n.best > 1 && is.null(point.est)){
+    if(!is.null(cl)){
+        fitted.matrix$y.hat <- parallel::parApply(cl, original.IVs, 1, function(z) NNS::NNS.distance(REGRESSION.POINT.MATRIX, dist.estimate = z, type = dist, k = n.best)[1])
+    } else {
+        fits <- data.table::data.table(original.IVs)
+        fits <- fits[, DISTANCES :=  NNS.distance(REGRESSION.POINT.MATRIX, dist.estimate = .SD, type = dist, k = n.best)[1], by = 1:nrow(original.IVs)]
+
+        fitted.matrix$y.hat <- as.numeric(unlist(fits$DISTANCES))
+    }
+
+    y.hat <- fitted.matrix$y.hat
+
+    if(!is.null(type)){
+        y.hat <- round(y.hat)
     }
   }
 
@@ -216,31 +242,24 @@ NNS.M.reg <- function (X_n, Y, factor.2.dummy = FALSE, order = NULL, stn = NULL,
       outsiders <- numeric()
       DISTANCES <- list()
 
-      ### PARALLEL
 
-      if (is.null(ncores)) {
-        num_cores <- as.integer(detectCores() / 2) - 1
+
+
+      distances <- data.table::data.table(point.est)
+
+      if(!is.null(cl)){
+        DISTANCES <- parallel::parApply(cl, distances, 1, function(z) NNS::NNS.distance(REGRESSION.POINT.MATRIX, dist.estimate = z, type = dist, k = n.best)[1])
+
+        stopCluster(cl)
+        registerDoSEQ()
       } else {
-        num_cores <- ncores
+
+        distances <- distances[, DISTANCES :=  NNS.distance(REGRESSION.POINT.MATRIX, dist.estimate = .SD, type = dist, k = n.best)[1], by = 1:nrow(point.est)]
+
+        DISTANCES <- as.numeric(unlist(distances$DISTANCES))
       }
 
 
-        cl <- makeCluster(num_cores)
-        registerDoParallel(cl)
-
-
-      DISTANCES <- foreach(i = 1:nrow(point.est),.packages = c("NNS","data.table", "dtw"))%dopar%{
-        NNS.distance(rpm = REGRESSION.POINT.MATRIX, dist.estimate = point.est[i,],
-                     type = dist, k = n.best)[1]
-      }
-
-
-
-      stopCluster(cl)
-      registerDoSEQ()
-
-      DISTANCES <- unlist(DISTANCES)
-
       lows <- do.call(pmin,as.data.frame(t(point.est))) < minimums
       highs <- do.call(pmax,as.data.frame(t(point.est))) > maximums
 
@@ -306,7 +325,7 @@ NNS.M.reg <- function (X_n, Y, factor.2.dummy = FALSE, order = NULL, stn = NULL,
 
   R2 <- (sum((y.hat - mean(original.DV)) * (original.DV - mean(original.DV))) ^ 2) / (sum((original.DV - mean(original.DV)) ^ 2) * sum((y.hat - mean(original.DV)) ^ 2))
 
-  fitted.matrix$residuals <- fitted.matrix$y.hat - fitted.matrix$y
+  fitted.matrix$residuals <- fitted.matrix$y - fitted.matrix$y.hat
 
   ### 3d plot
   if(plot && n == 2){
@@ -336,12 +355,12 @@ NNS.M.reg <- function (X_n, Y, factor.2.dummy = FALSE, order = NULL, stn = NULL,
       data.table::setkey(region.matrix, NNS.ID, min.x1, max.x1, min.x2, max.x2)
       region.matrix[ ,{
         rgl::quads3d(x = .(min.x1[1], min.x1[1], max.x1[1], max.x1[1]),
-                y = .(min.x2[1], max.x2[1], max.x2[1], min.x2[1]),
-                z = .(y.hat[1], y.hat[1], y.hat[1], y.hat[1]), col='pink', alpha=1)
+                     y = .(min.x2[1], max.x2[1], max.x2[1], min.x2[1]),
+                     z = .(y.hat[1], y.hat[1], y.hat[1], y.hat[1]), col='pink', alpha=1)
         if(identical(min.x1[1], max.x1[1]) || identical(min.x2[1], max.x2[1])){
           rgl::segments3d(x = .(min.x1[1], max.x1[1]),
-                     y = .(min.x2[1], max.x2[1]),
-                     z = .(y.hat[1], y.hat[1]), col = 'pink', alpha = 1)
+                          y = .(min.x2[1], max.x2[1]),
+                          z = .(y.hat[1], y.hat[1]), col = 'pink', alpha = 1)
         }
       }
       , by = NNS.ID]
